#pragma once

// Quelle: https://stackoverflow.com/questions/22432066/how-to-use-table-based-crc-16-code
//         http://srecord.sourceforge.net/crc16-ccitt.html
// Look-Up-Tabelle: https://www.mikrocontroller.net/attachment/91385/crc16.c

#include <cstdint>


// Klasse zur Berechnung einer CRC16-Prfsumme
// Benutzt eine Loop-Up-Tabelle
class CRC16GeneratorLookUp {
   // Polynom:   0x1021
   // Startwert: 0x1d0f

public:

   // Paket
   // ---------------------------------------

   // Generiert die CRC16 fr ein Datenpaket.
   uint16_t generateCRCFast(const uint8_t* data, size_t len);


   // Stream
   // ---------------------------------------

   // Gibt ein Byte an den Datenstrom ab und berechnet dabei fortlaufend die CRC.
   void streamCRCFast(uint8_t b);

   // Gibt die CRC an den Datenstrom weiter (LSB First) 
   // und initialisiert erneut die internen Variablen.
   uint16_t streamFin();

   // Funktion, die ein Byte an den Datenstrom abgibt.
   void (*executor)(uint8_t b) = dummyExcecutor;

protected:
   // Leere Funktion zur initialisierung der Variable 'executor'.
   static void dummyExcecutor(uint8_t b) {}

   // Startwert der CRC-Berechnung.
   const  uint16_t crcSeed = 0x1d0f;

   // Zwischenspeicher fr die fortlaufende Berechnung.
   uint16_t crc = crcSeed;
};